from wagtail_extensions.utils import first_true, nth, true_or_nth


def test_first_true_empty():
    out = first_true([])
    assert out == None


def test_first_true_default():
    out = first_true([], default='fish')
    assert out == 'fish'


def test_first_true_match():
    match = {'match': True}
    items = [{}, match]
    item = first_true(items, predicate=lambda x: x.get('match') == True)
    assert item == match


def test_first_true_nomatch():
    items = [{'match': False}, {'another': 5}]
    item = first_true(items, predicate=lambda x: x.get('match') == True)
    assert item == None


def test_true_or_nth_nomatch():
    first = {'match': False}
    items = [first, {'another': 5}]
    item = true_or_nth(items, predicate=lambda x: x.get('match') == True)
    assert item == first


def test_nth_empty():
    out = nth([], 2)
    assert out == None


def test_nth_default():
    out = nth([], 2, default=5)
    assert out == 5


def test_nth_works():
    out = nth([1, 2, 3, 4, 5], 2)
    assert out == 3



import datetime
import pytest
from unittest.mock import patch

from django.core.exceptions import ValidationError
from freezegun import freeze_time
from phonenumber_field.phonenumber import PhoneNumber

from wagtail.core.models import Page
from wagtail_extensions.blocks import (DepartmentBlock, LinkBlock,
                                       OpeningTimeBlock, OpeningTimesBlock,
                                       PhoneBlock)


@pytest.mark.django_db
@pytest.fixture
def page():
    # Homepage is created by Wagtail's initial migrations
    # But let's create our own child page for testing with.
    homepage = Page.objects.get(url_path='/home/')
    page = Page(title='A test page', slug="test")
    homepage.add_child(instance=page)
    return page


def test_department_block_clean_invalid():
    department = DepartmentBlock()
    with pytest.raises(ValidationError):
        department.clean({})


def test_department_block_clean_valid_with_both():
    department = DepartmentBlock()
    department.clean({'name':'Test', 'email':'foo@foo.com', 'phones': ['+447528712345']})


def test_department_block_to_python_empty():
    department = DepartmentBlock()
    department.to_python({})


def test_department_block_to_python_strip_empty_phonenumbers():
    department = DepartmentBlock()
    value = department.get_prep_value({'phones': ['', '+447528712345', '']})
    assert value['phones'] == ['+447528712345']


def test_link_block_with_url():
    block = LinkBlock()
    value = block.to_python({
        'link': [{'type': 'url', 'value': '/hello/'}]
    })

    assert value.link_url == '/hello/'
    assert value.link_text == '/hello/'


def test_link_block_with_url_and_text():
    block = LinkBlock()
    value = block.to_python({
        'text': 'Hello World',
        'link': [{'type': 'url', 'value': '/hello/'}]
    })
    assert value.link_url == '/hello/'
    assert value.link_text == 'Hello World'


def test_link_block_with_empty_string_text():
    block = LinkBlock()
    value = block.to_python({
        'text': '',
        'link': [{'type': 'url', 'value': '/hello/'}]
    })
    assert value.link_url == '/hello/'
    assert value.link_text == '/hello/'


def test_link_block_with_missing_streamblock():
    block = LinkBlock()
    value = block.to_python({
        'text': '',
        'link': []
    })
    assert value.link_url == ''
    assert value.link_text == ''


@pytest.mark.django_db
def test_link_block_with_page(page):
    block = LinkBlock()
    value = block.to_python({
        'link': [{'type': 'page', 'value': page.pk}]
    })

    assert value.link_url == page.url
    assert value.link_text == page.title


@pytest.mark.django_db
def test_link_block_with_page_and_text(page):
    block = LinkBlock()
    value = block.to_python({
        'text': 'Hello World',
        'link': [{'type': 'page', 'value': page.pk}]
    })
    assert value.link_url == page.url
    assert value.link_text == 'Hello World'


def test_link_block_clean_for_required():
    block = LinkBlock()
    value = block.to_python({
        'text': 'Hello World',
        'link': []      # This must not be empty if the field is required
    })
    with pytest.raises(ValidationError):
        block.clean(value)


def test_link_block_clean_for_not_required():
    block = LinkBlock(required=False)
    value = block.to_python({
        'text': 'Hello World',
        'link': []      # Can be empty if the field is not required
    })
    # This should not raise an exception
    block.clean(value)


@freeze_time("2017-01-01")
def test_openingtime_block_clean_date_in_past():
    openingtime = OpeningTimeBlock()
    with pytest.raises(ValidationError):
        openingtime.clean({'date': '2016-01-01'})


def test_openingtime_block_clean_end_before_start():
    openingtime = OpeningTimeBlock()
    with pytest.raises(ValidationError):
        openingtime.clean({'start': '20:00', 'end': '08:00', 'weekday': '1'})


def test_openingtime_block_clean_no_weekday_or_date():
    openingtime = OpeningTimeBlock()
    with pytest.raises(ValidationError):
        openingtime.clean({'start': '20:00', 'end': '08:00'})


@freeze_time("2017-01-01")
def test_openingtime_block_clean_valid():
    openingtime = OpeningTimeBlock()
    openingtime.clean({'start': '08:00', 'end': '20:00', 'date': '2017-01-01'})


def test_openingtime_block_to_python_empty():
    openingtime = OpeningTimeBlock()
    openingtime.to_python({'label': '', 'date': None, 'closed': False, 'start': None, 'end': None, 'weekday': ''})
    # Pass without error


def test_openingtime_block_to_python_cast_weekday():
    openingtime = OpeningTimeBlock()
    value = openingtime.to_python({'weekday': '5'})
    assert value['weekday'] == 5


def test_openingtime_block_to_python_public_label():
    openingtime = OpeningTimeBlock()
    value = openingtime.to_python({'weekday': '7'})
    assert value['label'] == OpeningTimeBlock.PUBLIC_LABEL


def test_openingtime_block_to_python_public_with_label():
    openingtime = OpeningTimeBlock()
    label = 'Easter sunday'
    value = openingtime.to_python({'weekday': '7', 'label': label})
    assert value['label'] == label


def test_openingtime_block_single_date_empty():
    assert OpeningTimeBlock.single_date({}) == False


def test_openingtime_block_single_date_with_date():
    assert OpeningTimeBlock.single_date({'date': 'some date'}) == True


def test_openingtime_block_single_date_public():
    assert OpeningTimeBlock.single_date({'weekday': 7}) == True


def test_openingtime_block_next_date_empty():
    assert OpeningTimeBlock.next_date({}) is None


@freeze_time("2017-12-13")
def test_openingtime_block_next_date_today():
    assert OpeningTimeBlock.next_date({'weekday': 2}) == datetime.date(2017, 12, 13)


@freeze_time("2017-12-13")
def test_openingtime_block_next_date_sunday():
    assert OpeningTimeBlock.next_date({'weekday': 6}) == datetime.date(2017, 12, 17)


@freeze_time("2017-12-13")
def test_openingtime_block_next_date_public():
    assert OpeningTimeBlock.next_date({'weekday': 7}) is None


def test_openingtimes_block_time_keyfunc_specific():
    openingtime = OpeningTimeBlock()
    value = openingtime.to_python({})
    with patch.object(openingtime, 'single_date', return_value=True):
        out = OpeningTimesBlock.time_keyfunc(value)
    assert out is value


def test_openingtimes_block_time_keyfunc_non_specific():
    value = OpeningTimeBlock().to_python({'closed': False, 'start': '5:00', 'end': '10:00'})
    out = OpeningTimesBlock.time_keyfunc(value)
    assert out == (False, datetime.time(5), datetime.time(10))


@patch('wagtail_extensions.blocks.groupby')
def test_openingtimes_block_group_times(mocked_groupby):
    value = {}
    mocked_groupby.return_value = [('first', [1, 4, 5]), ('second', [7, 10])]
    out = OpeningTimesBlock.group_times(value)

    assert out == [[1, 4, 5], [7, 10]]
    mocked_groupby.assert_called_once_with(value, OpeningTimesBlock.time_keyfunc)


def test_openingtimes_block_get_time_for_date_empty():
    assert OpeningTimesBlock.get_time_for_date(None, None) is None


def test_openingtimes_block_get_time_for_date_no_times():
    assert OpeningTimesBlock.get_time_for_date({}, datetime.date(2017, 12, 10)) is None


def test_openingtimes_block_get_time_for_date_times_date():
    match = {'date': datetime.date(2017, 12, 10)}
    value = {
        'times': [
            {'weekday': 4},
            match,
        ],
    }
    assert OpeningTimesBlock.get_time_for_date(value, datetime.date(2017, 12, 10)) == match


def test_openingtimes_block_get_time_for_date_times_weekday():
    match = {'weekday': 6}
    value = {
        'times': [
            {'weekday': 4},
            {'date': datetime.date(2017, 12, 10)},
            match,
        ],
    }
    assert OpeningTimesBlock.get_time_for_date(value, datetime.date(2017, 12, 17)) == match


def test_openingtimes_block_get_time_for_date_times_no_match():
    value = {
        'times': [
            {'weekday': 4},
            {'date': datetime.date(2017, 12, 10)},
            {'weekday': 2},
        ],
    }
    assert OpeningTimesBlock.get_time_for_date(value, datetime.date(2017, 12, 17)) is None


@freeze_time('2017-06-28')
def test_openingtimes_block_opening_today():
    openingtimes = OpeningTimesBlock
    with patch.object(openingtimes, 'get_time_for_date') as mocked_get:
        value = 'test'
        out = openingtimes.opening_today(value)
        mocked_get.assert_called_once_with(value, datetime.date(2017, 6, 28))
        assert out == mocked_get.return_value


def test_openingtimes_block_get_context():
    openingtimes = OpeningTimesBlock()
    value = {'times': [1, 5, 10]}
    with patch.object(openingtimes, 'group_times') as mocked_group,\
          patch.object(openingtimes, 'opening_today') as mocked_today:
        ctx = openingtimes.get_context(value)
        mocked_group.assert_called_once_with([1, 5, 10])
        mocked_today.assert_called_once_with(value)
        assert ctx['times'] == mocked_group.return_value
        assert ctx['today'] == mocked_today.return_value


def test_phone_block_get_prep_value():
    phone = PhoneBlock()
    number = PhoneNumber.from_string('+447528712345')
    number_str = phone.get_prep_value(number)
    assert number_str == '+447528712345'


def test_phone_block_to_python():
    phone = PhoneBlock()
    number = phone.to_python('+447528712345')
    assert number == PhoneNumber.from_string('+447528712345')


def test_phone_block_to_python_empty():
    phone = PhoneBlock()
    assert phone.to_python('') == ''



import pytest
from datetime import timedelta
from django.template import engines
from django.utils import timezone

from wagtail.core.models import Page
from wagtail_extensions.templatetags.wagtailextensions_tags import (
    bleachclean, page_menu_children, track_form_submission, menu)


@pytest.mark.django_db
@pytest.fixture
def page_tree():
    # Homepage is created by Wagtail's initial migrations
    # But let's create our own child page for testing with.
    root = Page.objects.get(url_path='/home/')
    page_1 = Page(title='A test page 1', slug="test_1", show_in_menus=True, live=True)
    page_2 = Page(title='A test page 2', slug="test_2", show_in_menus=True, live=True)
    page_3 = Page(title='A test page 3', slug="test_3", show_in_menus=False, live=True)
    page_4 = Page(title='A test page 4', slug="test_4", show_in_menus=True, live=False)
    page_1_1 = Page(title='A test child of page 1', slug="child_1", show_in_menus=True, live=True)
    root.add_child(instance=page_1)
    root.add_child(instance=page_2)
    root.add_child(instance=page_3)
    root.add_child(instance=page_4)
    page_1.add_child(instance=page_1_1)
    pages = [page_1, page_2, page_3, page_4, page_1_1]
    return (root, pages)


@pytest.fixture
def render_template():
    """
    Returns a helper function that takes a template string, and returns the
    rendered output.
    """
    template_engine = engines['django']
    def func(template_string):
        load_tags_string = '{% load wagtailextensions_tags %}'
        return template_engine.from_string(load_tags_string + template_string).render()
    return func


def test_bleanclean_cleandata():
    cleaned = bleachclean('Hello')

    assert cleaned == 'Hello'


def test_bleanclean_strips():
    cleaned = bleachclean('<script>evil</script>')

    assert cleaned == 'evil'


def test_track_form_submission(rf):
    request = rf.get('/')
    request.session = {
        'enquiry_form_submitted': timezone.now().strftime('%Y-%m-%d %H:%M %z')
    }

    ctx = track_form_submission(request)

    assert ctx == {'enquiry_form_submitted': True}
    assert request.session == {}


def test_track_form_submission_expired(rf):
    request = rf.get('/')
    request.session = {
        # this was submitted too long ago to qualify
        'enquiry_form_submitted': (timezone.now() - timedelta(seconds=3600)).strftime('%Y-%m-%d %H:%M %z')
    }

    ctx = track_form_submission(request)

    assert ctx == {'enquiry_form_submitted': False}
    assert request.session == {}


def test_track_form_submission_no_submission(rf):
    request = rf.get('/')
    request.session = {}

    ctx = track_form_submission(request)

    assert ctx == {'enquiry_form_submitted': False}


def test_track_form_invalid_session_data(rf):
    request = rf.get('/')
    request.session = {
        # this was submitted too long ago to qualify
        'enquiry_form_submitted': 'this is not a valid timestamp'
    }

    ctx = track_form_submission(request)

    assert ctx == {'enquiry_form_submitted': False}


@pytest.mark.django_db
def test_page_menu_children(page_tree):
    root, pages = page_tree
    out = page_menu_children(root, calling_page=None)
    assert len(out) == 2
    assert out[0].slug == 'test_1'
    assert out[0].active == False
    assert out[1].slug == 'test_2'
    assert out[1].active == False


@pytest.mark.django_db
def test_page_menu_children(page_tree):
    root, pages = page_tree
    out = page_menu_children(root, calling_page=pages[0])
    assert len(out) == 2
    assert out[0].active == True
    assert out[1].active == False


@pytest.mark.django_db
def test_menu_tag(page_tree, rf):
    root, pages = page_tree
    request = rf.get('test_1')
    out = menu({'request': request}, root, pages[0])
    assert out['calling_page'] == pages[0]
    assert out['request'] == request
    assert out['menuitems'][0].slug == 'test_1'
    assert out['menuitems'][0].active == True
    assert out['menuitems'][1].slug == 'test_2'
    assert len(out['menuitems'][0].children) == 1
    assert out['menuitems'][0].children[0].slug == 'child_1'


def test_metablock_with_no_modifications(render_template):
    template_string = '{% metablock %}Hello{% endmetablock %}'
    expected_output = 'Hello'
    output = render_template(template_string)
    assert output == expected_output


def test_metablock_with_modifications(render_template):
    template_string = """{% metablock %}
        The world's "fastest"

        <a href="">supercomputers</a> , period <-> Jane & John Doe.
    {% endmetablock %}"""
    expected_output = 'The world&#39;s &quot;fastest&quot; supercomputers, period &lt;-&gt; Jane &amp; John Doe.'
    output = render_template(template_string)
    assert output == expected_output



from unittest.mock import Mock, patch

from wagtail_extensions.forms import ContactForm


cleaned_data = {
    'email': 't@t.com',
    'message': 'Hi',
    'name': 'test',
    'subject_prefix': 'Admin'
}


def test_email_subject_prefix_added_to_ctx():
    form = ContactForm()
    form.cleaned_data = cleaned_data

    ctx = form.get_email_context()

    assert ctx['subject_prefix'] == 'Admin'


def test_go_to_with_enquiryemail():
    form = ContactForm()
    page = Mock()
    page.enquiry_email = 'e@e.com'
    to = form.get_to(page)

    assert to == ['e@e.com']


def test_go_to_without_enquiryemail():
    form = ContactForm()
    page = Mock()
    page.enquiry_email = ''
    to = form.get_to(page)

    assert to == ['admin@localhost.com']


@patch('wagtail_extensions.forms.ContactForm.send_email')
@patch('wagtail_extensions.forms.ContactForm.get_to')
def test_save(mocked_get_to, mocked_send_email):
    form = ContactForm()
    form.cleaned_data = cleaned_data
    page = Mock()
    page.enquiry_email = 'e@e.com'
    form.save(page)

    mocked_get_to.assert_called_once_with(page)
    mocked_send_email.assert_called_once_with(
        mocked_get_to.return_value,
        'wagtail_extensions/email/subject.txt',
        'wagtail_extensions/email/message.txt',
        ['t@t.com']
    )



import datetime
import pytest
from unittest import mock

from freezegun import freeze_time

from wagtail.core.models import Site
from wagtail_extensions.forms import ContactForm
from wagtail_extensions.mixins import ContactMixin
from wagtail_extensions.models import ContactSubmission

from tests.testproject.testapp.models import ContactDetailsTestSetting


@pytest.mark.django_db
@pytest.fixture
def contact_setting():
    setting = ContactDetailsTestSetting()
    setting.site = Site.objects.get(is_default_site=True)
    setting.save()
    return setting


@pytest.mark.django_db
def test_contact_details_primary_location_empty(contact_setting):
    assert contact_setting.primary_location == None


@pytest.mark.django_db
def test_contact_details_primary_location_found(contact_setting):
    match = {'primary': True, 'name': 'Test location'}
    contact_setting.locations = [
        ('location', {'primary': False}),
        ('location', {}),
        ('location', match),
    ]
    assert contact_setting.primary_location.value['name'] == 'Test location'


@pytest.mark.django_db
def test_contact_details_primary_department_empty(contact_setting):
    assert contact_setting.primary_department == None


@pytest.mark.django_db
def test_contact_details_primary_department_found(contact_setting):
    match = {'primary': True}
    contact_setting.locations = [
        ('location', {
            'primary': True,
            'departments': [
                {'primary': False},
                match,
                {},
            ],
        }),
    ]
    assert contact_setting.primary_department == match


@pytest.mark.django_db
def test_contact_details_primary_phone_empty(contact_setting):
    assert contact_setting.primary_phone == None


@pytest.mark.django_db
def test_contact_details_primary_phone_found(contact_setting):
    contact_setting.locations = [
        ('location', {
            'primary': True,
            'departments': [
                {'primary': True, 'phones': ['+447528712345', '+447528712346', '+447528712347']},
            ],
        })
    ]
    assert contact_setting.primary_phone == '+447528712345'


@pytest.mark.django_db
def test_contact_details_primary_opening_times_found(contact_setting):
    times = {'times': [{'label': 'My time'}]}

    contact_setting.locations = [
        ('location', {
            'primary': True,
            'opening_times': times,
        })
    ]

    assert contact_setting.primary_opening_times == times


@pytest.mark.django_db
def test_contact_details_primary_opening_times_none(contact_setting):
    assert contact_setting.primary_opening_times == None


@freeze_time("2017-06-06")
@pytest.mark.django_db
def test_contact_details_primary_opening_today_none(contact_setting):
    contact_setting.locations = [
        ('location', {
            'primary': True,
            'opening_times': {'times': [
                {'weekday': 3},
                {'date': datetime.date(2017, 6, 5)},
                {'weekday': 7},
            ]},
        })
    ]

    contact_setting.save()
    contact_setting.refresh_from_db()

    assert contact_setting.primary_opening_today == None


@freeze_time("2017-06-06")
@pytest.mark.django_db
def test_contact_details_primary_opening_today_date(contact_setting):
    match = datetime.date(2017, 6, 6)

    contact_setting.locations = [
        ('location', {
            'primary': True,
            'opening_times': {'times': [
                {'weekday': 1},
                {'date': match},
            ]},
        })
    ]

    contact_setting.save()
    contact_setting.refresh_from_db()

    assert contact_setting.primary_opening_today['date'] == match


@freeze_time("2017-06-10")
@pytest.mark.django_db
def test_contact_details_primary_opening_today_weekday(contact_setting):
    match = 5

    contact_setting.locations = [
        ('location', {
            'primary': True,
            'opening_times': {'times': [
                {'weekday': 3},
                {'date': datetime.date(2017, 6, 5)},
                {'weekday': 5},
            ]},
        })
    ]

    contact_setting.save()
    contact_setting.refresh_from_db()

    assert contact_setting.primary_opening_today['weekday'] == match


@pytest.mark.django_db
def test_contact_details_primary_opening_today_no_location(contact_setting):
    assert contact_setting.primary_opening_today == None


@freeze_time("2017-12-05")
def test_contact_details_get_opening_today_cache_key():
    out = ContactDetailsTestSetting.get_opening_today_cache_key()
    assert out == 'wagtail_extensions_opening_today_20171205'


@pytest.mark.django_db
def test_store_submission(rf):
    form_data = {
        'name': 'Alice',
        'email': 'alice@example.com',
        'message': 'Where is Bob?',
        'foobar': '',   # Honeypot field
    }
    request = rf.post('/', form_data)
    # Fake messages handler
    request._messages = mock.MagicMock()
    request.session = mock.MagicMock()
    page = ContactMixin()
    page.url = '/'
    page.serve(request)
    submission = ContactSubmission.objects.first()
    del form_data['foobar']
    assert submission.data == form_data


@pytest.mark.django_db
def test_disable_store_submission(rf):
    form_data = {
        'name': 'Alice',
        'email': 'alice@example.com',
        'message': 'Where is Bob?',
        'foobar': '',   # Honeypot field
    }
    request = rf.post('/', form_data)
    # Fake messages handler
    request._messages = mock.MagicMock()
    request.session = mock.MagicMock()
    page = ContactMixin()
    page.url = '/'
    page.store_submissions = False
    page.serve(request)
    assert ContactSubmission.objects.count() == 0


@pytest.mark.django_db
def test_submission_saves_tracker_to_session(rf):
    form_data = {
        'name': 'Alice',
        'email': 'alice@example.com',
        'message': 'Where is Bob?',
        'foobar': '',   # Honeypot field
    }
    request = rf.post('/', form_data)
    request.session = {}
    request._messages = mock.MagicMock()

    page = ContactMixin()
    page.url = '/'
    page.serve(request)
    assert 'enquiry_form_submitted' in request.session
